@@ -10,12 +10,23 @@ import re
 import shlex
 import subprocess
 from pathlib import Path
-from typing import Any, Dict, List, Tuple
+from typing import Any, Dict, List, Optional, Tuple
 
 from blight import util
-from blight.enums import BlightTool, BuildTool, CodeModel, CompilerStage, Lang, OptLevel, Std
+from blight.constants import COMPILER_FLAG_INJECTION_VARIABLES
+from blight.enums import (
+    BlightTool,
+    BuildTool,
+    CodeModel,
+    CompilerFamily,
+    CompilerStage,
+    Lang,
+    OptLevel,
+    Std,
+)
 from blight.exceptions import BlightError, BuildError, SkipRun
 from blight.protocols import CanonicalizedArgsProtocol, IndexedUndefinesProtocol, LangProtocol
+from blight.util import json_helper
 
 logger = logging.getLogger(__name__)
 
@@ -89,7 +100,7 @@ class Tool:
         self._cwd = Path(os.getcwd()).resolve()
         self._actions = util.load_actions()
         self._skip_run = False
-        self._action_results: Dict[str, Dict[str, Any]] = {}
+        self._action_results: Dict[str, Optional[Dict[str, Any]]] = {}
         self._journal_path = os.getenv("BLIGHT_JOURNAL_PATH")
 
     def _fixup_env(self) -> Dict[str, str]:
@@ -118,7 +129,7 @@ class Tool:
     def _commit_journal(self) -> None:
         if self.is_journaling():
             with util.flock_append(self._journal_path) as io:  # type: ignore
-                json.dump(self._action_results, io)
+                json.dump(self._action_results, io, default=json_helper)
                 # NOTE(ww): `json.dump` doesn't do this for us.
                 io.write("\n")
 
@@ -544,7 +555,7 @@ class ResponseFileMixin:
             )
 
         self._canonicalized_args = expanded_args
-        return self._canonicalized_args
+        return self._canonicalized_args  # type: ignore[no-any-return]
 
 
 class DefinesMixin:
@@ -569,10 +580,7 @@ class DefinesMixin:
                 continue
 
             # Both `-Uname` and `-U name` work in GCC and Clang.
-            if arg == "-U":
-                undefine = self.canonicalized_args[idx + 1]
-            else:
-                undefine = arg[2:]
+            undefine = self.canonicalized_args[idx + 1] if arg == "-U" else arg[2:]
 
             indexed_undefines[undefine] = idx
 
@@ -593,20 +601,15 @@ class DefinesMixin:
                 continue
 
             # Both `-Dname[=value]` and `-D name[=value]` work in GCC and Clang.
-            if arg == "-D":
-                define = self.canonicalized_args[idx + 1]
-            else:
-                define = arg[2:]
+            define = self.canonicalized_args[idx + 1] if arg == "-D" else arg[2:]
 
             components = define.split("=", 1)
             name = components[0]
-            if len(components) == 1:
-                # NOTE(ww): 1 is the default macro value.
-                # It's actually an integer at the preprocessor level, but we model everything
-                # as strings here to avoid complicating things.
-                value = "1"
-            else:
-                value = components[1]
+
+            # NOTE(ww): 1 is the default macro value.
+            # It's actually an integer at the preprocessor level, but we model everything
+            # as strings here to avoid complicating things.
+            value = "1" if len(components) == 1 else components[1]
 
             # Is this macro subsequently undefined? If so, don't include it in
             # the defines list.
@@ -625,7 +628,7 @@ class CodeModelMixin:
     """
 
     @property
-    def code_model(self: CanonicalizedArgsProtocol):
+    def code_model(self: CanonicalizedArgsProtocol) -> CodeModel:
         """
         Returns:
             A `blight.enums.CodeModel` value representing the tool's code model
@@ -708,11 +711,58 @@ class CompilerTool(
     Like `Tool`, `CompilerTool` cannot be instantiated directly.
     """
 
-    def __init__(self, args) -> None:
+    def __init__(self, args: List[str]) -> None:
         if self.__class__ == CompilerTool:
             raise NotImplementedError(f"can't instantiate {self.__class__.__name__} directly")
+
         super().__init__(args)
 
+        # #40 and #41: These should be handled in an overridden implementation
+        # of `canonicalized_args`.
+        injection_vars = COMPILER_FLAG_INJECTION_VARIABLES & self._env.keys()
+        if injection_vars:
+            logger.warning(f"not tracking compiler's own instrumentation: {injection_vars}")
+
+    @property
+    def family(self) -> CompilerFamily:
+        """
+        Returns:
+            A `blight.enums.CompilerFamily` value representing the "family" of compilers
+            that this tool belongs to.
+        """
+
+        # NOTE(ww): Both GCC and Clang support -### as an alias for -v, but
+        # with additional guarantees around argument quoting. Do other families support it?
+
+        result = subprocess.run([self.wrapped_tool(), "-###"], capture_output=True)
+
+        # If the command exited with an error, we're likely dealing with a frontend
+        # that doesn't understand `-###`.
+        if result.returncode != 0:
+            logger.warning("compiler fingerprint failed: frontend didn't recognize -###?")
+            # ...but even still, we can infer a bit from the error message.
+            if b"tcc: error" in result.stderr:
+                return CompilerFamily.Tcc
+            else:
+                return CompilerFamily.Unknown
+
+        # We expect the relevant parts of `-###` on stderr. The lack of any output
+        # again suggests that the frontend doesn't understand the flag.
+        if not result.stderr:
+            logger.warning("compiler fingerprint failed: frontend didn't produce output for -###?")
+            return CompilerFamily.Unknown
+
+        # Finally, we do some silly substring checks.
+        # TODO(ww): Better heuristics here?
+        if b"Apple clang version" in result.stderr:
+            return CompilerFamily.AppleLlvm
+        elif b"clang version" in result.stderr:
+            return CompilerFamily.MainlineLlvm
+        elif b"gcc version" in result.stderr:
+            return CompilerFamily.Gcc
+        else:
+            return CompilerFamily.Unknown
+
     @property
     def stage(self) -> CompilerStage:
         """
@@ -917,7 +967,7 @@ class INSTALL(Tool):
             prog=self.build_tool().value, add_help=False, allow_abbrev=False
         )
 
-        def add_flag(short: str, dest: str, **kwargs) -> None:
+        def add_flag(short: str, dest: str, **kwargs: Any) -> None:
             parser.add_argument(short, action="store_true", dest=dest, **kwargs)
 
         add_flag("-b", "overwrite")
@@ -954,7 +1004,7 @@ class INSTALL(Tool):
         Returns whether this `install` invocation is in "directory mode," i.e.
         is creating directories instead of installing files.
         """
-        return self._matches.directory_mode
+        return self._matches.directory_mode  # type: ignore[no-any-return]
 
     @property
     def inputs(self) -> List[str]:
@@ -977,7 +1027,7 @@ class INSTALL(Tool):
         # to determine which mode we're in.
         maybe_dir = self._cwd / self._matches.trailing[-1]
         if maybe_dir.is_dir():
-            return self._matches.trailing[0:-1]
+            return self._matches.trailing[0:-1]  # type: ignore[no-any-return]
         else:
             return [self._matches.trailing[0]]
 
@@ -989,7 +1039,7 @@ class INSTALL(Tool):
 
         # Directory mode: treat created directories as outputs.
         if self.directory_mode:
-            return self._matches.trailing
+            return self._matches.trailing  # type: ignore[no-any-return]
 
         # `install` requires at least two positionals outside of directory mode,
         # so this probably indicates an unknown GNUism like `--help`.
